@@ -53,7 +53,6 @@ import org.mortbay.jetty.HttpException;
 import org.mortbay.jetty.MimeTypes;
 import org.mortbay.jetty.Request;
 import org.mortbay.jetty.Server;
-import org.mortbay.jetty.webapp.WebAppClassLoader;
 import org.mortbay.log.Log;
 import org.mortbay.log.Logger;
 import org.mortbay.resource.Resource;
@@ -63,27 +62,27 @@ import org.mortbay.util.LazyList;
 import org.mortbay.util.Loader;
 import org.mortbay.util.URIUtil;
 
+
 /* ------------------------------------------------------------ */
 /** ContextHandler.
  * 
  * This handler wraps a call to handle by setting the context and
  * servlet path, plus setting the context classloader.
  * 
- * <p>
- * If the context init parameter "org.mortbay.jetty.servlet.ManagedAttributes"
- * is set to a coma separated list of names, then they are treated as context
- * attribute names, which if set as attributes are passed to the servers Container
- * so that they may be managed with JMX.
+ * Note. Because of http://bugs.sun.com/bugdatabase/view_bug.do?bug_id=4425695
+ * directly replacing war or jar files in a context is not supported.
+ * You should use classes instead of jars if they will change, or deploy
+ * a packed war file that gets extracted on deployment.
+ * 
  * 
  * @org.apache.xbean.XBean description="Creates a basic HTTP context"
  *
  * @author gregw
  *
  */
-public class ContextHandler extends HandlerWrapper implements Attributes, Server.Graceful
+public class ContextHandler extends HandlerWrapper implements Attributes
 {
     private static ThreadLocal __context=new ThreadLocal();
-    public static final String MANAGED_ATTRIBUTES = "org.mortbay.jetty.servlet.ManagedAttributes";
     
     /* ------------------------------------------------------------ */
     /** Get the current ServletContext implementation.
@@ -100,12 +99,13 @@ public class ContextHandler extends HandlerWrapper implements Attributes, Server
 
     protected SContext _scontext;
     
-    private AttributesMap _attributes;
-    private AttributesMap _contextAttributes;
+    private Attributes _attributes;
+    private Attributes _contextAttributes;
     private ClassLoader _classLoader;
-    private String _contextPath="/";
+    private String _contextPath;
     private Map _initParams;
     private String _displayName;
+    private String _docRoot;
     private Resource _baseResource;  
     private MimeTypes _mimeTypes;
     private Map _localeEncodingMap;
@@ -116,15 +116,12 @@ public class ContextHandler extends HandlerWrapper implements Attributes, Server
     private EventListener[] _eventListeners;
     private Logger _logger;
     private boolean _shutdown;
-    private boolean _allowNullPathInfo;
-    private int _maxFormContentSize=Integer.getInteger("org.mortbay.jetty.Request.maxFormContentSize",200000).intValue();
-    private boolean _compactPath=false;
+    private int _maxFormContentSize=Integer.getInteger("org.mortbay.jetty.Request.maxFormContentSize",200000).intValue();;
 
     private Object _contextListeners;
     private Object _contextAttributeListeners;
     private Object _requestListeners;
     private Object _requestAttributeListeners;
-    private Set _managedAttributes;
     
     /* ------------------------------------------------------------ */
     /**
@@ -176,24 +173,6 @@ public class ContextHandler extends HandlerWrapper implements Attributes, Server
     {
         return _scontext;
     }
-    
-    /* ------------------------------------------------------------ */
-    /**
-     * @return the allowNullPathInfo true if /context is not redirected to /context/
-     */
-    public boolean getAllowNullPathInfo()
-    {
-        return _allowNullPathInfo;
-    }
-
-    /* ------------------------------------------------------------ */
-    /**
-     * @param allowNullPathInfo  true if /context is not redirected to /context/
-     */
-    public void setAllowNullPathInfo(boolean allowNullPathInfo)
-    {
-        _allowNullPathInfo=allowNullPathInfo;
-    }
 
     /* ------------------------------------------------------------ */
     public void setServer(Server server)
@@ -210,6 +189,7 @@ public class ContextHandler extends HandlerWrapper implements Attributes, Server
         }
         else
             super.setServer(server); 
+        
     }
 
     /* ------------------------------------------------------------ */
@@ -221,21 +201,11 @@ public class ContextHandler extends HandlerWrapper implements Attributes, Server
      * matching virtual host name.
      * @param vhosts Array of virtual hosts that this context responds to. A
      * null host name or null/empty array means any hostname is acceptable.
-     * Host names may be String representation of IP addresses. Host names may
-     * start with '*.' to wildcard one level of names.
+     * Host names may String representation of IP addresses.
      */
-    public void setVirtualHosts( String[] vhosts )
+    public void setVirtualHosts(String[] vhosts)
     {
-        if ( vhosts == null )
-        {
-            _vhosts = vhosts;
-        } 
-        else 
-        {
-            _vhosts = new String[vhosts.length];
-            for ( int i = 0; i < vhosts.length; i++ )
-                _vhosts[i] = normalizeHostname( vhosts[i]);
-        }
+        _vhosts=vhosts;
     }
 
     /* ------------------------------------------------------------ */
@@ -248,7 +218,6 @@ public class ContextHandler extends HandlerWrapper implements Attributes, Server
      * @return Array of virtual hosts that this context responds to. A
      * null host name or empty array means any hostname is acceptable.
      * Host names may be String representation of IP addresses.
-     * Host names may start with '*.' to wildcard one level of names.
      */
     public String[] getVirtualHosts()
     {
@@ -317,7 +286,7 @@ public class ContextHandler extends HandlerWrapper implements Attributes, Server
      */
     public Enumeration getAttributeNames()
     {
-        return AttributesMap.getAttributeNamesCopy(_attributes);
+        return _attributes.getAttributeNames();
     }
     
     /* ------------------------------------------------------------ */
@@ -484,18 +453,13 @@ public class ContextHandler extends HandlerWrapper implements Attributes, Server
      */
     protected void doStart() throws Exception
     {
-        if (_contextPath==null)
-            throw new IllegalStateException("Null contextPath");
-        
         _logger=Log.getLogger(getDisplayName()==null?getContextPath():getDisplayName());
         ClassLoader old_classloader=null;
         Thread current_thread=null;
-        SContext old_context=null;
-
+        Object old_context=null;
         _contextAttributes=new AttributesMap();
         try
         {
-            
             // Set the classloader
             if (_classLoader!=null)
             {
@@ -508,7 +472,7 @@ public class ContextHandler extends HandlerWrapper implements Attributes, Server
             if (_mimeTypes==null)
                 _mimeTypes=new MimeTypes();
             
-            old_context=(SContext)__context.get();
+            old_context=__context.get();
             __context.set(_scontext);
             
             if (_errorHandler==null)
@@ -516,7 +480,6 @@ public class ContextHandler extends HandlerWrapper implements Attributes, Server
             
             startContext();
             
-           
         }
         finally
         {
@@ -548,23 +511,6 @@ public class ContextHandler extends HandlerWrapper implements Attributes, Server
                 ((ServletContextListener)LazyList.get(_contextListeners, i)).contextInitialized(event);
             }
         }
-
-        String managedAttributes = (String)_initParams.get(MANAGED_ATTRIBUTES);
-        if (managedAttributes!=null)
-        {
-            _managedAttributes=new HashSet();
-            String[] attributes = managedAttributes.toString().split(",");
-	    for (int  i=0;i<attributes.length;i++)
-                _managedAttributes.add(attributes[i]);
-
-            Enumeration e = _scontext.getAttributeNames();
-            while(e.hasMoreElements())
-            {
-                String name = (String)e.nextElement();
-                Object value = _scontext.getAttribute(name);
-                setManagedAttribute(name,value);
-            }
-        }       
     }
     
     /* ------------------------------------------------------------ */
@@ -576,7 +522,7 @@ public class ContextHandler extends HandlerWrapper implements Attributes, Server
         ClassLoader old_classloader=null;
         Thread current_thread=null;
 
-        SContext old_context=(SContext)__context.get();
+        Object old_context=__context.get();
         __context.set(_scontext);
         try
         {
@@ -602,13 +548,6 @@ public class ContextHandler extends HandlerWrapper implements Attributes, Server
 
             if (_errorHandler!=null)
                 _errorHandler.stop();
-            
-            Enumeration e = _scontext.getAttributeNames();
-            while(e.hasMoreElements())
-            {
-                String name = (String)e.nextElement();
-                setManagedAttribute(name,null);
-            }
         }
         finally
         {
@@ -652,21 +591,12 @@ public class ContextHandler extends HandlerWrapper implements Attributes, Server
             // Check the vhosts
             if (_vhosts!=null && _vhosts.length>0)
             {
-                String vhost = normalizeHostname( request.getServerName());
-
+                String vhost=request.getServerName();
                 boolean match=false;
                 
                 // TODO non-linear lookup
                 for (int i=0;!match && i<_vhosts.length;i++)
-                {
-                    String contextVhost = _vhosts[i];
-                    if(contextVhost==null) continue;
-                    if(contextVhost.startsWith("*.")) {
-                        // wildcard only at the beginning, and only for one additional subdomain level
-                        match=contextVhost.regionMatches(true,2,vhost,vhost.indexOf(".")+1,contextVhost.length()-2);
-                    } else
-                        match=contextVhost.equalsIgnoreCase(vhost);
-                }
+                    match=_vhosts[i]!=null && _vhosts[i].equalsIgnoreCase(vhost);
                 if (!match)
                     return;
             }
@@ -682,25 +612,18 @@ public class ContextHandler extends HandlerWrapper implements Attributes, Server
             // Nope - so check the target.
             if (dispatch==REQUEST)
             {
-                if (_compactPath)
-                    target=URIUtil.compactPath(target);
-                
                 if (target.equals(_contextPath))
                 {
-                    if (!_allowNullPathInfo && !target.endsWith(URIUtil.SLASH))
+                    target=_contextPath;
+                    if (!target.endsWith("/"))
                     {
                         base_request.setHandled(true);
                         if (request.getQueryString()!=null)
-                            response.sendRedirect(URIUtil.addPaths(request.getRequestURI(),URIUtil.SLASH)+"?"+request.getQueryString());
+                            response.sendRedirect(target+"/?"+request.getQueryString());
                         else 
-                            response.sendRedirect(URIUtil.addPaths(request.getRequestURI(),URIUtil.SLASH));
+                            response.sendRedirect(target+"/");
                         return;
                     }
-                    if (_contextPath.length()>1)
-                    {
-                        target=URIUtil.SLASH;
-                        request.setAttribute("org.mortbay.jetty.nullPathInfo",target);
-                    }
                 }
                 else if (target.startsWith(_contextPath) && (_contextPath.length()==1 || target.charAt(_contextPath.length())=='/'))
                 {
@@ -821,7 +744,6 @@ public class ContextHandler extends HandlerWrapper implements Attributes, Server
      */
     public void removeAttribute(String name)
     {
-        setManagedAttribute(name,null);
         _attributes.removeAttribute(name);
     }
 
@@ -834,7 +756,6 @@ public class ContextHandler extends HandlerWrapper implements Attributes, Server
      */
     public void setAttribute(String name, Object value)
     {
-        setManagedAttribute(name,value);
         _attributes.setAttribute(name,value);
     }
     
@@ -844,54 +765,14 @@ public class ContextHandler extends HandlerWrapper implements Attributes, Server
      */
     public void setAttributes(Attributes attributes)
     {
-        if (attributes instanceof AttributesMap)
-        {
-            _attributes = (AttributesMap)attributes;
-            Enumeration e = _attributes.getAttributeNames();
-            while (e.hasMoreElements())
-            {
-                String name = (String)e.nextElement();
-                setManagedAttribute(name,attributes.getAttribute(name));
-            }
-        }
-        else
-        {
-            _attributes=new AttributesMap();
-            Enumeration e = attributes.getAttributeNames();
-            while (e.hasMoreElements())
-            {
-                String name = (String)e.nextElement();
-                Object value=attributes.getAttribute(name);
-                setManagedAttribute(name,value);
-                _attributes.setAttribute(name,value);
-            }
-        }
+        _attributes = attributes;
     }
 
     /* ------------------------------------------------------------ */
     public void clearAttributes()
     {
-        Enumeration e = _attributes.getAttributeNames();
-        while (e.hasMoreElements())
-        {
-            String name = (String)e.nextElement();
-            setManagedAttribute(name,null);
-        }
         _attributes.clearAttributes();
     }
-
-    /* ------------------------------------------------------------ */
-    private void setManagedAttribute(String name, Object value)
-    {   
-        if (_managedAttributes!=null && _managedAttributes.contains(name))
-        {
-            Object o =_scontext.getAttribute(name);
-            if (o!=null)
-                getServer().getContainer().removeBean(o);
-            if (value!=null)
-                getServer().getContainer().addBean(value);
-        }
-    }
     
     /* ------------------------------------------------------------ */
     /**
@@ -938,8 +819,6 @@ public class ContextHandler extends HandlerWrapper implements Attributes, Server
     public void setDisplayName(String servletContextName)
     {
         _displayName = servletContextName;
-        if (_classLoader!=null && _classLoader instanceof WebAppClassLoader)
-            ((WebAppClassLoader)_classLoader).setName(servletContextName);
     }
     
     /* ------------------------------------------------------------ */
@@ -971,6 +850,23 @@ public class ContextHandler extends HandlerWrapper implements Attributes, Server
     public void setBaseResource(Resource base) 
     {
         _baseResource=base;
+        _docRoot=null;
+        
+        try
+        {
+            File file=_baseResource.getFile();
+            if (file!=null)
+            {
+                _docRoot=file.getCanonicalPath();
+                if (_docRoot.endsWith(File.pathSeparator))
+                    _docRoot=_docRoot.substring(0,_docRoot.length()-1);
+            }
+        }
+        catch (Exception e)
+        {
+            Log.warn(e);
+            throw new IllegalArgumentException(base.toString());
+        }
     }
 
     /* ------------------------------------------------------------ */
@@ -1062,24 +958,6 @@ public class ContextHandler extends HandlerWrapper implements Attributes, Server
     }
 
 
-    /* ------------------------------------------------------------ */
-    /**
-     * @return True if URLs are compacted to replace multiple '/'s with a single '/'
-     */
-    public boolean isCompactPath()
-    {
-        return _compactPath;
-    }
-
-    /* ------------------------------------------------------------ */
-    /**
-     * @param compactPath True if URLs are compacted to replace multiple '/'s with a single '/'
-     */
-    public void setCompactPath(boolean compactPath)
-    {
-        _compactPath=compactPath;
-    }
-
     /* ------------------------------------------------------------ */
     public String toString()
     {
@@ -1134,7 +1012,7 @@ public class ContextHandler extends HandlerWrapper implements Attributes, Server
      */
     public Resource getResource(String path) throws MalformedURLException
     {
-        if (path==null || !path.startsWith(URIUtil.SLASH))
+        if (path==null || !path.startsWith("/"))
             throw new MalformedURLException(path);
         
         if (_baseResource==null)
@@ -1167,8 +1045,8 @@ public class ContextHandler extends HandlerWrapper implements Attributes, Server
             
             if (resource!=null && resource.exists())
             {
-                if (!path.endsWith(URIUtil.SLASH))
-                    path=path+URIUtil.SLASH;
+                if (!path.endsWith("/"))
+                    path=path+"/";
                 
                 String[] l=resource.list();
                 if (l!=null)
@@ -1184,7 +1062,7 @@ public class ContextHandler extends HandlerWrapper implements Attributes, Server
         {
             Log.ignore(e);
         }
-        return Collections.emptySet();
+        return Collections.EMPTY_SET;
     }
 
     
@@ -1285,29 +1163,19 @@ public class ContextHandler extends HandlerWrapper implements Attributes, Server
          */
         public String getRealPath(String path)
         {
-            if(path==null)
+            if (_docRoot==null)
                 return null;
-            if(path.length()==0)
-                path = URIUtil.SLASH;
-            else if(path.charAt(0)!='/')
-                path = URIUtil.SLASH + path;
-                
-            try
-            {
-                Resource resource=ContextHandler.this.getResource(path);
-                if(resource!=null)
-                {
-                    File file = resource.getFile();
-                    if (file!=null)
-                        return file.getCanonicalPath();
-                }
-            }
-            catch (Exception e)
-            {
-                Log.ignore(e);
-            }
             
-            return null;
+            if (path==null)
+                return null;
+            path=URIUtil.canonicalPath(path);
+            
+            if (!path.startsWith("/"))
+                path="/"+path;
+            if (File.separatorChar!='/')
+                path =path.replace('/', File.separatorChar);
+            
+            return _docRoot+path;
         }
 
         /* ------------------------------------------------------------ */
@@ -1365,7 +1233,7 @@ public class ContextHandler extends HandlerWrapper implements Attributes, Server
          */
         public String getServerInfo()
         {
-            return "jetty/"+Server.getVersion();
+            return "jetty-"+getServer().getVersion();
         }
 
         /* ------------------------------------------------------------ */
@@ -1447,7 +1315,7 @@ public class ContextHandler extends HandlerWrapper implements Attributes, Server
         public synchronized Object getAttribute(String name)
         {
             Object o = ContextHandler.this.getAttribute(name);
-            if (o==null && _contextAttributes!=null)
+            if (o==null)
                 o=_contextAttributes.getAttribute(name);
             return o;
         }
@@ -1459,13 +1327,10 @@ public class ContextHandler extends HandlerWrapper implements Attributes, Server
         public synchronized Enumeration getAttributeNames()
         {
             HashSet set = new HashSet();
-            if (_contextAttributes!=null)
-            {
-            	Enumeration e = _contextAttributes.getAttributeNames();
-            	while(e.hasMoreElements())
-            		set.add(e.nextElement());
-            }
-            Enumeration e = _attributes.getAttributeNames();
+            Enumeration e = _contextAttributes.getAttributeNames();
+            while(e.hasMoreElements())
+                set.add(e.nextElement());
+            e = ContextHandler.this.getAttributeNames();
             while(e.hasMoreElements())
                 set.add(e.nextElement());
             
@@ -1478,15 +1343,9 @@ public class ContextHandler extends HandlerWrapper implements Attributes, Server
          */
         public synchronized void setAttribute(String name, Object value)
         {
-            
             if (_contextAttributes==null)
-            {
-            	// Set it on the handler
-            	ContextHandler.this.setAttribute(name, value);
                 return;
-            }
-
-            setManagedAttribute(name,value);
+            
             Object old_value=_contextAttributes==null?null:_contextAttributes.getAttribute(name);
             
             if (value==null)
@@ -1519,15 +1378,6 @@ public class ContextHandler extends HandlerWrapper implements Attributes, Server
          */
         public synchronized void removeAttribute(String name)
         {
-            setManagedAttribute(name,null);
-            
-            if (_contextAttributes==null)
-            {
-            	// Set it on the handler
-            	_attributes.removeAttribute(name);
-                return;
-            }
-            
             Object old_value=_contextAttributes.getAttribute(name);
             _contextAttributes.removeAttribute(name);
             if (old_value!=null)
@@ -1561,29 +1411,20 @@ public class ContextHandler extends HandlerWrapper implements Attributes, Server
          */
         public String getContextPath()
         {
-            if ((_contextPath != null) && _contextPath.equals(URIUtil.SLASH))
+            if ((_contextPath != null) && _contextPath.equals("/"))
                 return "";
             
             return _contextPath;
         }
 
+
         /* ------------------------------------------------------------ */
         public String toString()
         {
-            return "ServletContext@"+Integer.toHexString(hashCode())+"{"+(getContextPath().equals("")?URIUtil.SLASH:getContextPath())+","+getBaseResource()+"}";
+            return "ServletContext@"+Integer.toHexString(hashCode())+"{"+(getContextPath().equals("")?"/":getContextPath())+","+getBaseResource()+"}";
         }
+
     }
 
-    /* ------------------------------------------------------------ */
-    private String normalizeHostname( String host )
-    {
-        if ( host == null )
-            return null;
-        
-        if ( host.endsWith( "." ) )
-            return host.substring( 0, host.length() -1);
-      
-            return host;
-    }
 
 }
