@@ -21,7 +21,6 @@ import java.nio.channels.SelectionKey;
 import java.nio.channels.ServerSocketChannel;
 import java.nio.channels.SocketChannel;
 
-import org.mortbay.io.Buffer;
 import org.mortbay.io.Connection;
 import org.mortbay.io.nio.SelectChannelEndPoint;
 import org.mortbay.io.nio.SelectorManager;
@@ -65,11 +64,14 @@ import org.mortbay.util.ajax.Continuation;
 public class SelectChannelConnector extends AbstractNIOConnector 
 {
     private transient ServerSocketChannel _acceptChannel;
+    private long _lowResourcesConnections;
+    private long _lowResourcesMaxIdleTime;
 
     private SelectorManager _manager = new SelectorManager()
     {
         protected SocketChannel acceptChannel(SelectionKey key) throws IOException
         {
+            // TODO handle max connections
             SocketChannel channel = ((ServerSocketChannel)key.channel()).accept();
             if (channel==null)
                 return null;
@@ -79,24 +81,26 @@ public class SelectChannelConnector extends AbstractNIOConnector
             return channel;
         }
 
-        protected boolean dispatch(Runnable task) throws IOException
+        public boolean dispatch(Runnable task) throws IOException
         {
             return getThreadPool().dispatch(task);
         }
 
         protected void endPointClosed(SelectChannelEndPoint endpoint)
         {
+            // TODO handle max connections and low resources
             connectionClosed((HttpConnection)endpoint.getConnection());
         }
 
         protected void endPointOpened(SelectChannelEndPoint endpoint)
         {
+            // TODO handle max connections and low resources
             connectionOpened((HttpConnection)endpoint.getConnection());
         }
 
         protected Connection newConnection(SocketChannel channel,SelectChannelEndPoint endpoint)
         {
-            return new HttpConnection(SelectChannelConnector.this,endpoint,getServer());
+            return SelectChannelConnector.this.newConnection(channel,endpoint);
         }
 
         protected SelectChannelEndPoint newEndPoint(SocketChannel channel, SelectSet selectSet, SelectionKey sKey) throws IOException
@@ -120,25 +124,34 @@ public class SelectChannelConnector extends AbstractNIOConnector
         _manager.doSelect(acceptorID);
     }
     
-    /* ------------------------------------------------------------ */
-    public void stopAccept(int acceptorID) throws Exception
-    {
-        _manager.doStop(acceptorID);
-    }
-    
     /* ------------------------------------------------------------ */
     public void close() throws IOException
     {
-        if (_acceptChannel != null)
-            _acceptChannel.close();
-        _acceptChannel = null;
-
+        synchronized(this)
+        {
+            if(_manager.isRunning())
+            {
+                try
+                {
+                    _manager.stop();
+                }
+                catch (Exception e)
+                {
+                    Log.warn(e);
+                }
+            }
+            if (_acceptChannel != null)
+                _acceptChannel.close();
+            _acceptChannel = null;
+        }
     }
     
     /* ------------------------------------------------------------------------------- */
     public void customize(org.mortbay.io.EndPoint endpoint, Request request) throws IOException
     {
-        ((ConnectorEndPoint)endpoint).cancelIdle();
+        ConnectorEndPoint cep = ((ConnectorEndPoint)endpoint);
+        cep.cancelIdle();
+        request.setTimeStamp(cep.getSelectSet().getNow());
         super.customize(endpoint, request);
     }
     
@@ -170,9 +183,12 @@ public class SelectChannelConnector extends AbstractNIOConnector
     /* ------------------------------------------------------------------------------- */
     public int getLocalPort()
     {
-        if (_acceptChannel==null || !_acceptChannel.isOpen())
-            return -1;
-        return _acceptChannel.socket().getLocalPort();
+        synchronized(this)
+        {
+            if (_acceptChannel==null || !_acceptChannel.isOpen())
+                return -1;
+            return _acceptChannel.socket().getLocalPort();
+        }
     }
 
     /* ------------------------------------------------------------ */
@@ -187,18 +203,22 @@ public class SelectChannelConnector extends AbstractNIOConnector
     /* ------------------------------------------------------------ */
     public void open() throws IOException
     {
-        if (_acceptChannel == null)
+        synchronized(this)
         {
-            // Create a new server socket and set to non blocking mode
-            _acceptChannel = ServerSocketChannel.open();
-            _acceptChannel.configureBlocking(false);
+            if (_acceptChannel == null)
+            {
+                // Create a new server socket
+                _acceptChannel = ServerSocketChannel.open();
 
-            // Bind the server socket to the local host and port
-            InetSocketAddress addr = getHost()==null?new InetSocketAddress(getPort()):new InetSocketAddress(getHost(),getPort());
-            _acceptChannel.socket().bind(addr,getAcceptQueueSize());
+                // Bind the server socket to the local host and port
+                _acceptChannel.socket().setReuseAddress(getReuseAddress());
+                InetSocketAddress addr = getHost()==null?new InetSocketAddress(getPort()):new InetSocketAddress(getHost(),getPort());
+                _acceptChannel.socket().bind(addr,getAcceptQueueSize());
 
-            // Register accepts on the server socket with the selector.
-            _manager.register(_acceptChannel,SelectionKey.OP_ACCEPT);
+                // Set to non blocking mode
+                _acceptChannel.configureBlocking(false);
+                
+            }
         }
     }
 
@@ -221,6 +241,66 @@ public class SelectChannelConnector extends AbstractNIOConnector
         super.setMaxIdleTime(maxIdleTime);
     }
 
+
+    /* ------------------------------------------------------------ */
+    /**
+     * @return the lowResourcesConnections
+     */
+    public long getLowResourcesConnections()
+    {
+        return _lowResourcesConnections;
+    }
+
+    /* ------------------------------------------------------------ */
+    /**
+     * Set the number of connections, which if exceeded places this manager in low resources state.
+     * This is not an exact measure as the connection count is averaged over the select sets.
+     * @param lowResourcesConnections the number of connections
+     * @see {@link #setLowResourcesMaxIdleTime(long)}
+     */
+    public void setLowResourcesConnections(long lowResourcesConnections)
+    {
+        _lowResourcesConnections=lowResourcesConnections;
+    }
+
+    /* ------------------------------------------------------------ */
+    /**
+     * @return the lowResourcesMaxIdleTime
+     */
+    public long getLowResourcesMaxIdleTime()
+    {
+        return _lowResourcesMaxIdleTime;
+    }
+
+    /* ------------------------------------------------------------ */
+    /**
+     * Set the period in ms that a connection is allowed to be idle when this there are more
+     * than {@link #getLowResourcesConnections()} connections.  This allows the server to rapidly close idle connections
+     * in order to gracefully handle high load situations.
+     * @param lowResourcesMaxIdleTime the period in ms that a connection is allowed to be idle when resources are low.
+     * @see {@link #setMaxIdleTime(long)}
+     * @deprecated use {@link #setLowResourceMaxIdleTime(int)}
+     */
+    public void setLowResourcesMaxIdleTime(long lowResourcesMaxIdleTime)
+    {
+        _lowResourcesMaxIdleTime=lowResourcesMaxIdleTime;
+        super.setLowResourceMaxIdleTime((int)lowResourcesMaxIdleTime); // TODO fix the name duplications
+    }
+
+    /* ------------------------------------------------------------ */
+    /**
+     * Set the period in ms that a connection is allowed to be idle when this there are more
+     * than {@link #getLowResourcesConnections()} connections.  This allows the server to rapidly close idle connections
+     * in order to gracefully handle high load situations.
+     * @param lowResourcesMaxIdleTime the period in ms that a connection is allowed to be idle when resources are low.
+     * @see {@link #setMaxIdleTime(long)}
+     */
+    public void setLowResourceMaxIdleTime(int lowResourcesMaxIdleTime)
+    {
+        _lowResourcesMaxIdleTime=lowResourcesMaxIdleTime;
+        super.setLowResourceMaxIdleTime(lowResourcesMaxIdleTime); 
+    }
+    
     /* ------------------------------------------------------------ */
     /*
      * @see org.mortbay.jetty.AbstractConnector#doStart()
@@ -229,7 +309,11 @@ public class SelectChannelConnector extends AbstractNIOConnector
     {
         _manager.setSelectSets(getAcceptors());
         _manager.setMaxIdleTime(getMaxIdleTime());
+        _manager.setLowResourcesConnections(getLowResourcesConnections());
+        _manager.setLowResourcesMaxIdleTime(getLowResourcesMaxIdleTime());
         _manager.start();
+        open();
+        _manager.register(_acceptChannel);
         super.doStart();
     }
 
@@ -238,8 +322,7 @@ public class SelectChannelConnector extends AbstractNIOConnector
      * @see org.mortbay.jetty.AbstractConnector#doStop()
      */
     protected void doStop() throws Exception
-    {
-        _manager.stop();
+    {        
         super.doStop();
     }
 
@@ -248,7 +331,12 @@ public class SelectChannelConnector extends AbstractNIOConnector
     {
         return new ConnectorEndPoint(channel,selectSet,key);
     }
-    
+
+    /* ------------------------------------------------------------------------------- */
+    protected Connection newConnection(SocketChannel channel,SelectChannelEndPoint endpoint)
+    {
+        return new HttpConnection(SelectChannelConnector.this,endpoint,getServer());
+    }
 
     /* ------------------------------------------------------------ */
     /* ------------------------------------------------------------ */
@@ -263,28 +351,39 @@ public class SelectChannelConnector extends AbstractNIOConnector
 
         public void close() throws IOException
         {
-            RetryContinuation continuation = (RetryContinuation) ((HttpConnection)getConnection()).getRequest().getContinuation();
-            if (continuation != null && continuation.isPending())
-                continuation.reset();
+            Connection con=getConnection();
+            if (con instanceof HttpConnection)
+            {
+                RetryContinuation continuation = (RetryContinuation) ((HttpConnection)getConnection()).getRequest().getContinuation();
+                if (continuation != null && continuation.isPending())
+                    continuation.reset();
+            }
 
             super.close();
         }
 
+        /* ------------------------------------------------------------ */
         public void undispatch()
         {
-            RetryContinuation continuation = (RetryContinuation) ((HttpConnection)getConnection()).getRequest().getContinuation();
-
-            if (continuation != null)
+            Connection con=getConnection();
+            if (con instanceof HttpConnection)
             {
-                // We have a continuation
-                Log.debug("continuation {}", continuation);
-                if (!continuation.schedule())
+                RetryContinuation continuation = (RetryContinuation) ((HttpConnection)getConnection()).getRequest().getContinuation();
+
+                if (continuation != null)
+                {
+                    // We have a continuation
+                    Log.debug("continuation {}", continuation);
+                    if (continuation.undispatch())
+                        super.undispatch();
+                }
+                else
+                {
                     super.undispatch();
+                }
             }
             else
-            {
                 super.undispatch();
-            }
         }
     }
 
@@ -298,24 +397,10 @@ public class SelectChannelConnector extends AbstractNIOConnector
         Object _object;
         boolean _pending = false;   // waiting for resume or timeout
         boolean _resumed = false;   // resume called.
-        boolean _scheduled =false;  // Either dispatched or timeout set.
+        boolean _parked =false;     // end point dispatched, but undispatch called.
         RetryRequest _retry;
         long _timeout;
-        
-        public void expire()
-        {
-            boolean redispatch=false;
-            synchronized (this)
-            {
-                redispatch=_scheduled && _pending && !_resumed;
-            }
-            if (redispatch)
-            {
-                _endPoint.scheduleIdle();  // TODO maybe not needed?
-                _endPoint.getSelectSet().addChange(this);
-                _endPoint.getSelectSet().wakeup();
-            }
-        }
+
         
         public Object getObject()
         {
@@ -339,7 +424,7 @@ public class SelectChannelConnector extends AbstractNIOConnector
 
         public boolean isResumed()
         {
-            return _pending;
+            return _resumed;
         }
 
         public void reset()
@@ -348,7 +433,7 @@ public class SelectChannelConnector extends AbstractNIOConnector
             {
                 _resumed = false;
                 _pending = false;
-                _scheduled = false;
+                _parked = false;
             }
             
             synchronized (_endPoint.getSelectSet())
@@ -356,6 +441,39 @@ public class SelectChannelConnector extends AbstractNIOConnector
                 this.cancel();   
             }
         } 
+
+        public boolean suspend(long timeout)
+        {
+            boolean resumed=false;
+            synchronized (this)
+            {
+                resumed=_resumed;
+                _resumed=false;
+                _new = false;
+                if (!_pending && !resumed && timeout >= 0)
+                {
+                    _pending=true;
+                    _parked = false;
+                    _timeout = timeout;
+                    if (_retry==null)
+                     _retry = new RetryRequest();
+                    throw _retry;
+                }
+                
+                // here only if suspend called on pending continuation.
+                // acts like a reset
+                _resumed = false;
+                _pending = false;
+                _parked =false;
+            }
+
+            synchronized (_endPoint.getSelectSet())
+            {
+                this.cancel();   
+            }
+
+            return resumed;
+        }
         
         public void resume()
         {
@@ -365,7 +483,8 @@ public class SelectChannelConnector extends AbstractNIOConnector
                 if (_pending && !isExpired())
                 {
                     _resumed = true;
-                    redispatch=_scheduled;
+                    redispatch=_parked;
+                    _parked=false;
                 }
             }
 
@@ -384,24 +503,42 @@ public class SelectChannelConnector extends AbstractNIOConnector
             }
         }
         
+        public void expire()
+        {
+            boolean redispatch=false;
+            synchronized (this)
+            {
+                redispatch=_parked && _pending && !_resumed;
+                _parked=false;
+            }
+            if (redispatch)
+            {
+                _endPoint.scheduleIdle();  // TODO maybe not needed?
+                _endPoint.getSelectSet().addChange(this);
+                _endPoint.getSelectSet().wakeup();
+            }
+        }
+
+        
         public void run()
         {
             _endPoint.run();
         }
         
-        /* schedule continuation.
-         * Called when a run exits.  
+        /* undispatch continuation.
+         * Called when an endppoint is undispatched.  
          * Either sets timeout or dispatches if already resumed or expired */
-        public boolean schedule()
+        public boolean undispatch()
         {
             boolean redispatch=false;
         
             synchronized (this)
             {
                 if (!_pending)
-                    return false;
-                _scheduled = true;
+                    return true;
+                
                 redispatch=isExpired() || _resumed;
+                _parked=!redispatch;
             }
             
             if (redispatch)
@@ -409,49 +546,31 @@ public class SelectChannelConnector extends AbstractNIOConnector
                 _endPoint.scheduleIdle();
                 _endPoint.getSelectSet().addChange(this);
             }
-            else
+            else if (_timeout>0)
                 _endPoint.getSelectSet().scheduleTimeout(this,_timeout);
             
             _endPoint.getSelectSet().wakeup();
-            return true;
+            return false;
         }
 
         public void setObject(Object object)
         {
             _object = object;
         }
-
-        public boolean suspend(long timeout)
+        
+        public String toString()
         {
-            boolean resumed=false;
             synchronized (this)
             {
-                resumed=_resumed;
-                _resumed=false;
-                _new = false;
-                if (!_pending && !resumed && timeout >= 0)
-                {
-                    _pending=true;
-                    _scheduled = false;
-                    _timeout = timeout;
-                    if (_retry==null)
-                     _retry = new RetryRequest();
-                    throw _retry;
-                }
-                
-                // here only if suspend called on pending continuation.
-                // acts like a reset
-                _resumed = false;
-                _pending = false;
+                return "RetryContinuation@"+hashCode()+
+                (_new?",new":"")+
+                (_pending?",pending":"")+
+                (_resumed?",resumed":"")+
+                (isExpired()?",expired":"")+
+                (_parked?",parked":"");
             }
-
-            synchronized (_endPoint.getSelectSet())
-            {
-                this.cancel();   
-            }
-
-            return resumed;
         }
+
     }
 
 }
