@@ -27,7 +27,6 @@ import javax.servlet.ServletContext;
 import javax.servlet.http.Cookie;
 import javax.servlet.http.HttpServletRequest;
 import javax.servlet.http.HttpSession;
-import javax.servlet.http.HttpSessionActivationListener;
 import javax.servlet.http.HttpSessionAttributeListener;
 import javax.servlet.http.HttpSessionBindingEvent;
 import javax.servlet.http.HttpSessionBindingListener;
@@ -41,6 +40,7 @@ import org.mortbay.jetty.Server;
 import org.mortbay.jetty.SessionIdManager;
 import org.mortbay.jetty.SessionManager;
 import org.mortbay.jetty.handler.ContextHandler;
+import org.mortbay.log.Log;
 import org.mortbay.util.LazyList;
 
 /* ------------------------------------------------------------ */
@@ -67,7 +67,6 @@ public abstract class AbstractSessionManager extends AbstractLifeCycle implement
     private static final HttpSessionContext __nullSessionContext=new NullSessionContext();
 
     private boolean _usingCookies=true;
-    
     /* ------------------------------------------------------------ */
     // Setting of max inactive interval for new sessions
     // -1 means no timeout
@@ -91,7 +90,6 @@ public abstract class AbstractSessionManager extends AbstractLifeCycle implement
     protected String _sessionPath;
     protected int _maxCookieAge=-1;
     protected int _refreshCookieAge;
-    protected boolean _nodeIdInSessionId;
 
     /* ------------------------------------------------------------ */
     public AbstractSessionManager()
@@ -103,7 +101,7 @@ public abstract class AbstractSessionManager extends AbstractLifeCycle implement
     {
         long now=System.currentTimeMillis();
 
-        Session s = ((SessionIf)session).getSession();
+        Session s =(Session)session;
         s.access(now);
         
         // Do we need to refresh the cookie?
@@ -141,8 +139,6 @@ public abstract class AbstractSessionManager extends AbstractLifeCycle implement
     /* ------------------------------------------------------------ */
     public void complete(HttpSession session)
     {
-        Session s = ((SessionIf)session).getSession();
-        s.complete();
     }
 
     /* ------------------------------------------------------------ */
@@ -151,6 +147,8 @@ public abstract class AbstractSessionManager extends AbstractLifeCycle implement
         _context=ContextHandler.getCurrentContext();
         _loader=Thread.currentThread().getContextClassLoader();
 
+        newSessionMap();
+
         if (_sessionIdManager==null)
         {
             Server server=getSessionHandler().getServer();
@@ -175,8 +173,8 @@ public abstract class AbstractSessionManager extends AbstractLifeCycle implement
         tmp=_context.getInitParameter(SessionManager.__SessionURLProperty);
         if (tmp!=null)
         {
-            _sessionURL=(tmp==null||"none".equals(tmp))?null:tmp;
-            _sessionURLPrefix=(tmp==null||"none".equals(tmp))?null:(";"+_sessionURL+"=");
+            _sessionURL=tmp;
+            _sessionURLPrefix=";"+_sessionURL+"=";
         }
 
         // set up the max session cookie age if it isn't already
@@ -228,21 +226,20 @@ public abstract class AbstractSessionManager extends AbstractLifeCycle implement
     }
 
     /* ------------------------------------------------------------ */
-    public HttpSession getHttpSession(String nodeId)
+    public HttpSession getHttpSession(String id)
     {
-        String cluster_id = getIdManager().getClusterId(nodeId);
+        int dot=id.lastIndexOf('.');
+        String cluster_id=(dot>0)?id.substring(0,dot):id;
         
         synchronized (this)
         {
             Session session = getSession(cluster_id);
             
-            if (session!=null && !session.getNodeId().equals(nodeId))
+            if (session!=null && !session.getId().equals(id))
                 session.setIdChanged(true);
             return session;
         }
     }
-
-    /* ------------------------------------------------------------ */
     /* ------------------------------------------------------------ */
     /**
      * @return Returns the metaManager used for cross context session management
@@ -315,8 +312,7 @@ public abstract class AbstractSessionManager extends AbstractLifeCycle implement
     {
         if (isUsingCookies())
         {
-            String id = getNodeId(session);
-            Cookie cookie=getHttpOnly()?new HttpOnlyCookie(_sessionCookie,id):new Cookie(_sessionCookie,id);
+            Cookie cookie=getHttpOnly()?new HttpOnlyCookie(_sessionCookie,session.getId()):new Cookie(_sessionCookie,session.getId());
 
             cookie.setPath((contextPath==null||contextPath.length()==0)?"/":contextPath);
             cookie.setMaxAge(getMaxCookieAge());
@@ -386,22 +382,7 @@ public abstract class AbstractSessionManager extends AbstractLifeCycle implement
     /* ------------------------------------------------------------ */
     public boolean isValid(HttpSession session)
     {
-        Session s = ((SessionIf)session).getSession();
-        return s.isValid();
-    }
-    
-    /* ------------------------------------------------------------ */
-    public String getClusterId(HttpSession session)
-    {
-        Session s = ((SessionIf)session).getSession();
-        return s.getClusterId();
-    }
-    
-    /* ------------------------------------------------------------ */
-    public String getNodeId(HttpSession session)
-    {
-        Session s = ((SessionIf)session).getSession();
-        return s.getNodeId();
+        return ((Session)session).isValid();
     }
 
     /* ------------------------------------------------------------ */
@@ -517,20 +498,14 @@ public abstract class AbstractSessionManager extends AbstractLifeCycle implement
         _sessionHandler=sessionHandler;
     }
 
-    /* ------------------------------------------------------------ */
     public void setSessionPath(String path)
     {
         _sessionPath=path;
     }
 
-    /* ------------------------------------------------------------ */
-    /** Set the session ID URL parameter name
-     * @param param The parameter name for session id URL rewriting (null or "none" for no rewriting).
-     */
-    public void setSessionURL(String param)
+    public void setSessionURL(String url)
     {
-        _sessionURL=(param==null||"none".equals(param))?null:param;
-        _sessionURLPrefix=(param==null||"none".equals(param))?null:(";"+_sessionURL+"=");
+        _sessionURL=url;
     }
     /* ------------------------------------------------------------ */
     /**
@@ -562,12 +537,8 @@ public abstract class AbstractSessionManager extends AbstractLifeCycle implement
                     this._maxSessions=getSessions();
             }
         }
-
-        if (!created)
-        {
-            session.didActivate();
-        }
-        else if (_sessionListeners!=null)
+        
+        if (created && _sessionListeners!=null)
         {
             HttpSessionEvent event=new HttpSessionEvent(session);
             for (int i=0; i<LazyList.size(_sessionListeners); i++)
@@ -581,7 +552,7 @@ public abstract class AbstractSessionManager extends AbstractLifeCycle implement
      * @param idInCluster The session ID in the cluster, stripped of any worker name.
      * @return A Session or null if none exists.
      */
-    public abstract Session getSession(String idInCluster);
+    protected abstract Session getSession(String idInCluster);
 
     protected abstract void invalidateSessions();
 
@@ -595,23 +566,9 @@ public abstract class AbstractSessionManager extends AbstractLifeCycle implement
     protected abstract Session newSession(HttpServletRequest request);
     
 
-
-    /* ------------------------------------------------------------ */
-    /**
-     * @return true if the cluster node id (worker id) is returned as part of the session id by {@link HttpSession#getId()}. Default is false. 
-     */
-    public boolean isNodeIdInSessionId()
+    // TODO delete this 
+    protected final void newSessionMap()
     {
-        return _nodeIdInSessionId;
-    }
-
-    /* ------------------------------------------------------------ */
-    /**
-     * @param nodeIdInSessionId true if the cluster node id (worker id) will be returned as part of the session id by {@link HttpSession#getId()}. Default is false.
-     */
-    public void setNodeIdInSessionId(boolean nodeIdInSessionId)
-    {
-        _nodeIdInSessionId=nodeIdInSessionId;
     }
 
     /* ------------------------------------------------------------ */
@@ -620,53 +577,27 @@ public abstract class AbstractSessionManager extends AbstractLifeCycle implement
      * @param invalidate True if {@link HttpSessionListener#sessionDestroyed(HttpSessionEvent)} and
      * {@link SessionIdManager#invalidateAll(String)} should be called.
      */
-    public void removeSession(HttpSession session, boolean invalidate)
+    protected void removeSession(Session session, boolean invalidate)
     {
-        Session s = ((SessionIf)session).getSession();
-        removeSession(s,invalidate);
-    }
-
-    /* ------------------------------------------------------------ */
-    /** Remove session from manager 
-     * @param session The session to remove
-     * @param invalidate True if {@link HttpSessionListener#sessionDestroyed(HttpSessionEvent)} and
-     * {@link SessionIdManager#invalidateAll(String)} should be called.
-     */
-    public void removeSession(Session session, boolean invalidate)
-    {
-        // Remove session from context and global maps
-        synchronized (_sessionIdManager)
-        {
-            boolean removed = false;
-            
-            synchronized (this)
-            {
-                //take this session out of the map of sessions for this context
-                if (getSession(session.getClusterId()) != null)
-                {
-                    removed = true;
-                    removeSession(session.getClusterId());
-                }
-            }   
-            
-            if (removed)
-            {
-                // Remove session from all context and global id maps
-                _sessionIdManager.removeSession(session);
-                if (invalidate)
-                    _sessionIdManager.invalidateAll(session.getClusterId());
-            }
-        }
-        
         if (invalidate && _sessionListeners!=null)
         {
             HttpSessionEvent event=new HttpSessionEvent(session);
             for (int i=LazyList.size(_sessionListeners); i-->0;)
                 ((HttpSessionListener)LazyList.get(_sessionListeners,i)).sessionDestroyed(event);
         }
-        if (!invalidate)
+        
+        // Remove session from context and global maps
+        synchronized (_sessionIdManager)
         {
-            session.willPassivate();
+            String id=session.getClusterId();
+            _sessionIdManager.removeSession(session);
+            
+            synchronized (this)
+            {
+                removeSession(id);
+            }
+            if (invalidate)
+                _sessionIdManager.invalidateAll(id);
         }
     }
 
@@ -705,19 +636,6 @@ public abstract class AbstractSessionManager extends AbstractLifeCycle implement
         }
     }
 
-    /* ------------------------------------------------------------ */
-    /* ------------------------------------------------------------ */
-    /* ------------------------------------------------------------ */
-    /**
-     * Interface that any session wrapper should implement so that
-     * SessionManager may access the Jetty session implementation.
-     *
-     */
-    public interface SessionIf extends HttpSession
-    {
-        public Session getSession();
-    }
-    
     /* ------------------------------------------------------------ */
     /* ------------------------------------------------------------ */
     /* ------------------------------------------------------------ */
@@ -729,21 +647,18 @@ public abstract class AbstractSessionManager extends AbstractLifeCycle implement
      * @author gregw
      *
      */
-    public abstract class Session implements SessionIf, Serializable
+    public abstract class Session implements HttpSession, Serializable
     {
-        protected final String _clusterId;
-        protected final String _nodeId;
-        protected boolean _idChanged;
-        protected final long _created;
-        protected long _cookieSet;
-        protected long _accessed;
-        protected long _lastAccessed;
-        protected boolean _invalid;
-        protected boolean _doInvalidate;
-        protected long _maxIdleMs=_dftMaxIdleSecs*1000;
-        protected boolean _newSession;
-        protected Map _values;
-        protected int _requests;
+        final String _clusterId;
+        final String _id;
+        boolean _idChanged;
+        final long _created;
+        long _cookieSet;
+        long _accessed;
+        boolean _invalid;
+        long _maxIdleMs=_dftMaxIdleSecs*1000;
+        boolean _newSession;
+        Map _values;
 
         /* ------------------------------------------------------------- */
         protected Session(HttpServletRequest request)
@@ -751,9 +666,8 @@ public abstract class AbstractSessionManager extends AbstractLifeCycle implement
             _newSession=true;
             _created=System.currentTimeMillis();
             _clusterId=_sessionIdManager.newSessionId(request,_created);
-            _nodeId=_sessionIdManager.getNodeId(_clusterId,request);
+            _id=getId(request);
             _accessed=_created;
-            _requests=1;
         }
 
         /* ------------------------------------------------------------- */
@@ -761,31 +675,41 @@ public abstract class AbstractSessionManager extends AbstractLifeCycle implement
         {
             _created=created;
             _clusterId=clusterId;
-            _nodeId=_sessionIdManager.getNodeId(_clusterId,null);
+            _id=getId(null);
             _accessed=_created;
         }
 
-        /* ------------------------------------------------------------- */
-        public Session getSession()
-        {
-            return this;
-        }
-        
         /* ------------------------------------------------------------- */
         protected void initValues() 
         {
             _values=newAttributeMap();
         }
 
+        /* ------------------------------------------------------------ */
+        /** Get the session ID with any worker ID.
+         * 
+         * @param request
+         * @return sessionId plus any worker ID.
+         */
+        protected String getId(HttpServletRequest request) 
+        {
+            String worker=request==null?null:(String)request.getAttribute("org.mortbay.http.ajp.JVMRoute");
+            if (worker!=null) 
+                return _clusterId+'.'+worker; 
+            
+            if (_sessionIdManager.getWorkerName()!=null) 
+                return _clusterId+'.'+_sessionIdManager.getWorkerName();
+           
+            return _clusterId;
+        }
+
         /* ------------------------------------------------------------ */
         public synchronized Object getAttribute(String name)
         {
             if (_invalid)
                 throw new IllegalStateException();
-
             if (null == _values)
                 return null;
-            
             return _values.get(name);
         }
 
@@ -812,22 +736,10 @@ public abstract class AbstractSessionManager extends AbstractLifeCycle implement
             return _created;
         }
 
-        /* ------------------------------------------------------------ */
+        /* ------------------------------------------------------------- */
         public String getId() throws IllegalStateException
         {
-            return _nodeIdInSessionId?_nodeId:_clusterId;
-        }
-
-        /* ------------------------------------------------------------- */
-        protected String getNodeId()
-        {
-            return _nodeId;
-        }
-        
-        /* ------------------------------------------------------------- */
-        protected String getClusterId()
-        {
-            return _clusterId;
+            return _id;
         }
 
         /* ------------------------------------------------------------- */
@@ -835,7 +747,7 @@ public abstract class AbstractSessionManager extends AbstractLifeCycle implement
         {
             if (_invalid)
                 throw new IllegalStateException();
-            return _lastAccessed;
+            return _accessed;
         }
 
         /* ------------------------------------------------------------- */
@@ -891,89 +803,39 @@ public abstract class AbstractSessionManager extends AbstractLifeCycle implement
             return (String[])_values.keySet().toArray(a);
         }
 
-        /* ------------------------------------------------------------ */
-        protected void access(long time)
-        {
-            synchronized(this)
-            {
-                _newSession=false;
-                _lastAccessed=_accessed;
-                _accessed=time;
-                _requests++;
-            }
-        }
-
-        /* ------------------------------------------------------------ */
-        protected void complete()
-        {
-            synchronized(this)
-            {
-                _requests--;
-                if (_doInvalidate && _requests<=0  )
-                    doInvalidate();
-            }
-        }
-        
-        
-        /* ------------------------------------------------------------- */
-        protected void timeout() throws IllegalStateException
-        {
-            // remove session from context and invalidate other sessions with same ID.
-            removeSession(this,true);
-
-            // Notify listeners and unbind values
-            synchronized (this)
-            {
-                if (_requests<=0)
-                    doInvalidate();
-                else
-                    _doInvalidate=true;
-            }
-        }
-        
         /* ------------------------------------------------------------- */
         public void invalidate() throws IllegalStateException
         {
-            // remove session from context and invalidate other sessions with same ID.
-            removeSession(this,true);
-            doInvalidate();
-        }
-        
-        /* ------------------------------------------------------------- */
-        protected void doInvalidate() throws IllegalStateException
-        {
+            if (Log.isDebugEnabled())
+                Log.debug("Invalidate session "+getId());
             try
             {
+                // remove session from context and invalidate other sessions with same ID.
+                removeSession(this,true);
+                
                 // Notify listeners and unbind values
-                if (_invalid)
-                    throw new IllegalStateException();
-
-                while (_values!=null && _values.size()>0)
+                synchronized (this)
                 {
-                    ArrayList keys;
-                    synchronized (this)
+                    if (_invalid)
+                        throw new IllegalStateException();
+
+                    if (_values!=null)
                     {
-                        keys=new ArrayList(_values.keySet());
-                    }
-
-                    Iterator iter=keys.iterator();
-                    while (iter.hasNext())
-                    {
-                        String key=(String)iter.next();
-
-                        Object value;
-                        synchronized (this)
+                        Iterator iter=_values.keySet().iterator();
+                        while (iter.hasNext())
                         {
-                            value=_values.remove(key);
-                        }
-                        unbindValue(key,value);
+                            String key=(String)iter.next();
+                            Object value=_values.get(key);
+                            iter.remove();
+                            unbindValue(key,value);
 
-                        if (_sessionAttributeListeners!=null)
-                        {
-                            HttpSessionBindingEvent event=new HttpSessionBindingEvent(this,key,value);
+                            if (_sessionAttributeListeners!=null)
+                            {
+                                HttpSessionBindingEvent event=new HttpSessionBindingEvent(this,key,value);
 
-                            for (int i=0; i<LazyList.size(_sessionAttributeListeners); i++)
-                                ((HttpSessionAttributeListener)LazyList.get(_sessionAttributeListeners,i)).attributeRemoved(event);
+                                for (int i=0; i<LazyList.size(_sessionAttributeListeners); i++)
+                                    ((HttpSessionAttributeListener)LazyList.get(_sessionAttributeListeners,i)).attributeRemoved(event);
+                            }
                         }
                     }
                 }
@@ -1098,6 +960,13 @@ public abstract class AbstractSessionManager extends AbstractLifeCycle implement
             return this.getClass().getName()+":"+getId()+"@"+hashCode();
         }
 
+        /* ------------------------------------------------------------ */
+        protected void access(long time)
+        {
+            _newSession=false;
+            _accessed=time;
+        }
+
         /* ------------------------------------------------------------- */
         /** If value implements HttpSessionBindingListener, call valueBound() */
         protected void bindValue(java.lang.String name, Object value)
@@ -1106,6 +975,12 @@ public abstract class AbstractSessionManager extends AbstractLifeCycle implement
                 ((HttpSessionBindingListener)value).valueBound(new HttpSessionBindingEvent(this,name));
         }
 
+        /* ------------------------------------------------------------- */
+        protected String getClusterId()
+        {
+            return _clusterId;
+        }
+
         /* ------------------------------------------------------------ */
         protected boolean isValid()
         {
@@ -1128,35 +1003,6 @@ public abstract class AbstractSessionManager extends AbstractLifeCycle implement
             if (value!=null&&value instanceof HttpSessionBindingListener)
                 ((HttpSessionBindingListener)value).valueUnbound(new HttpSessionBindingEvent(this,name));
         }
-
-        /* ------------------------------------------------------------- */
-        protected synchronized void willPassivate() 
-        {
-            HttpSessionEvent event = new HttpSessionEvent(this);
-            for (Iterator iter = _values.values().iterator(); iter.hasNext();) 
-            {
-                Object value = iter.next();
-                if (value instanceof HttpSessionActivationListener) 
-                {
-                    HttpSessionActivationListener listener = (HttpSessionActivationListener) value;
-                    listener.sessionWillPassivate(event);
-                }
-            }
-        }
-
-        /* ------------------------------------------------------------- */
-        protected synchronized void didActivate() 
-        {
-            HttpSessionEvent event = new HttpSessionEvent(this);
-            for (Iterator iter = _values.values().iterator(); iter.hasNext();) 
-            {
-                Object value = iter.next();
-                if (value instanceof HttpSessionActivationListener) 
-                {
-                    HttpSessionActivationListener listener = (HttpSessionActivationListener) value;
-                    listener.sessionDidActivate(event);
-                }
-            }
-        }
     }
+
 }
